==> sknlp-0.1.27-py3-none-any.whl: sknlp/module/discriminators/bert_discriminator.py <==
from typing import Any, Dict, Sequence, Optional

import tensorflow as tf
import tensorflow.keras.backend as K
from tensorflow.keras.layers import Dropout
import pandas as pd

from sknlp.layers import MLPLayer, BertLayer, BertPairPreprocessingLayer
from sknlp.vocab import Vocab
from sknlp.data import BertSimilarityDataset
from sknlp.module.text2vec import Bert2vec
from .deep_discriminator import DeepDiscriminator


class BertDiscriminator(DeepDiscriminator):
    def __init__(
        self,
        segmenter: Optional[str] = None,
        embedding_size: int = 100,
        use_batch_normalization: bool = True,
        max_sequence_length: int = 120,
        num_fc_layers: int = 2,
        fc_hidden_size: int = 256,
        fc_activation: str = "tanh",
        fc_momentum: float = 0.9,
        fc_epsilon: float = 1e-5,
        output_dropout: float = 0.5,
        text2vec: Optional[Bert2vec] = None,
        **kwargs
    ) -> None:
        super().__init__(
            segmenter=segmenter,
            algorithm="bert",
            embedding_size=embedding_size,
            use_batch_normalization=use_batch_normalization,
            max_sequence_length=max_sequence_length,
            text2vec=text2vec,
            **kwargs
        )
        self.num_fc_layers = num_fc_layers
        self.fc_hidden_size = fc_hidden_size
        self.fc_activation = fc_activation
        self.fc_momentum = fc_momentum
        self.fc_epsilon = fc_epsilon
        self.output_dropout = output_dropout
        self.inputs = [
            tf.keras.Input(shape=(), dtype=tf.string, name="text_input"),
            tf.keras.Input(shape=(), dtype=tf.string, name="context_input"),
        ]

    def create_dataset_from_df(
        self, df: pd.DataFrame, vocab: Vocab, segmenter: str, labels: Sequence[str]
    ) -> BertSimilarityDataset:
        return BertSimilarityDataset(
            vocab,
            list(labels),
            df=df,
            max_length=self.max_sequence_length,
        )

    def build_encode_layer(self, inputs: tf.Tensor) -> tf.Tensor:
        preprocessing_layer = BertPairPreprocessingLayer(
            self.text2vec.vocab.sorted_tokens
        )
        token_ids, type_ids = preprocessing_layer(inputs)
        _, _, cls = self.text2vec([token_ids, type_ids])
        if self.output_dropout:
            return Dropout(self.output_dropout, name="embedding_dropout")(cls)
        return cls

    def build_output_layer(self, inputs: tf.Tensor) -> tf.Tensor:
        return MLPLayer(
            self.num_fc_layers,
            hidden_size=self.fc_hidden_size,
            output_size=self.num_classes,
            activation=self.fc_activation,
            batch_normalization=self.use_batch_normalization,
            momentum=self.fc_momentum,
            epsilon=self.fc_epsilon,
            name="mlp",
        )(inputs)

    def get_config(self) -> Dict[str, Any]:
        return {**super().get_config(), "output_dropout": self.output_dropout}

    def get_custom_objects(self) -> Dict[str, Any]:
        return {
            **super().get_custom_objects(),
            "MLPLayer": MLPLayer,
            "BertLayer": BertLayer,
            "BertPairPreprocessingLayer": BertPairPreprocessingLayer,
        }

==> sknlp-0.3.3-py3-none-any.whl: sknlp/module/discriminators/bert_discriminator.py <==
from typing import Any, Dict, Sequence, Optional

import tensorflow as tf
from tensorflow.keras.layers import Dropout

from sknlp.layers import BertPairPreprocessingLayer
from sknlp.data import BertSimilarityDataset
from sknlp.module.text2vec import Bert2vec
from .deep_discriminator import DeepDiscriminator


class BertDiscriminator(DeepDiscriminator):
    dataset_class = BertSimilarityDataset

    def __init__(
        self,
        classes: Sequence[str] = ("相似度",),
        max_sequence_length: int = 120,
        num_fc_layers: int = 2,
        fc_hidden_size: int = 256,
        fc_activation: str = "tanh",
        dropout: float = 0.5,
        text2vec: Optional[Bert2vec] = None,
        **kwargs
    ) -> None:
        super().__init__(
            classes=classes,
            algorithm="bert",
            max_sequence_length=max_sequence_length,
            text2vec=text2vec,
            **kwargs
        )
        self.num_fc_layers = num_fc_layers
        self.fc_hidden_size = fc_hidden_size
        self.fc_activation = fc_activation
        self.dropout = dropout
        self.inputs = [
            tf.keras.Input(shape=(), dtype=tf.string, name="text_input"),
            tf.keras.Input(shape=(), dtype=tf.string, name="context_input"),
        ]

    def build_encode_layer(self, inputs: tf.Tensor) -> tf.Tensor:
        preprocessing_layer = BertPairPreprocessingLayer(
            self.text2vec.vocab.sorted_tokens
        )
        token_ids, type_ids = preprocessing_layer(inputs)
        if self.dropout:
            self.text2vec.update_dropout(dropout=self.dropout)
        _, cls = self.text2vec([token_ids, type_ids])
        if self.dropout:
            return Dropout(self.dropout, name="embedding_dropout")(cls)
        return cls

    def get_config(self) -> Dict[str, Any]:
        return {**super().get_config(), "dropout": self.dropout}
